@@ -16,7 +16,6 @@
 import os
 import time
 import traceback
-from datetime import datetime
 from swift import gettext_ as _
 from xml.etree.cElementTree import Element, SubElement, tostring
 
@@ -24,26 +23,54 @@ from eventlet import Timeout
 
 import swift.common.db
 from swift.container.backend import ContainerBroker, DATADIR
+from swift.container.replicator import ContainerReplicatorRpc
 from swift.common.db import DatabaseAlreadyExists
 from swift.common.container_sync_realms import ContainerSyncRealms
 from swift.common.request_helpers import get_param, get_listing_content_type, \
     split_and_validate_path, is_sys_or_user_meta
 from swift.common.utils import get_logger, hash_path, public, \
-    normalize_timestamp, storage_directory, validate_sync_to, \
+    Timestamp, storage_directory, validate_sync_to, \
     config_true_value, json, timing_stats, replication, \
-    override_bytes_from_content_type
-from swift.common.constraints import CONTAINER_LISTING_LIMIT, \
-    check_mount, check_float, check_utf8
+    override_bytes_from_content_type, get_log_line
+from swift.common.constraints import check_mount, valid_timestamp, check_utf8
+from swift.common import constraints
 from swift.common.bufferedhttp import http_connect
 from swift.common.exceptions import ConnectionTimeout
-from swift.common.db_replicator import ReplicatorRpc
 from swift.common.http import HTTP_NOT_FOUND, is_success
+from swift.common.storage_policy import POLICIES, POLICY_INDEX
 from swift.common.swob import HTTPAccepted, HTTPBadRequest, HTTPConflict, \
     HTTPCreated, HTTPInternalServerError, HTTPNoContent, HTTPNotFound, \
     HTTPPreconditionFailed, HTTPMethodNotAllowed, Request, Response, \
     HTTPInsufficientStorage, HTTPException, HeaderKeyDict
 
 
+def gen_resp_headers(info, is_deleted=False):
+    """
+    Convert container info dict to headers.
+    """
+    # backend headers are always included
+    headers = {
+        'X-Backend-Timestamp': Timestamp(info.get('created_at', 0)).internal,
+        'X-Backend-PUT-Timestamp': Timestamp(info.get(
+            'put_timestamp', 0)).internal,
+        'X-Backend-DELETE-Timestamp': Timestamp(
+            info.get('delete_timestamp', 0)).internal,
+        'X-Backend-Status-Changed-At': Timestamp(
+            info.get('status_changed_at', 0)).internal,
+        POLICY_INDEX: info.get('storage_policy_index', 0),
+    }
+    if not is_deleted:
+        # base container info on deleted containers is not exposed to client
+        headers.update({
+            'X-Container-Object-Count': info.get('object_count', 0),
+            'X-Container-Bytes-Used': info.get('bytes_used', 0),
+            'X-Timestamp': Timestamp(info.get('created_at', 0)).normal,
+            'X-PUT-Timestamp': Timestamp(
+                info.get('put_timestamp', 0)).normal,
+        })
+    return headers
+
+
 class ContainerController(object):
     """WSGI Controller for the container server."""
 
@@ -74,7 +101,7 @@ class ContainerController(object):
             h.strip()
             for h in conf.get('allowed_sync_hosts', '127.0.0.1').split(',')
             if h.strip()]
-        self.replicator_rpc = ReplicatorRpc(
+        self.replicator_rpc = ContainerReplicatorRpc(
             self.root, DATADIR, ContainerBroker, self.mount_check,
             logger=self.logger)
         self.auto_create_account_prefix = \
@@ -102,6 +129,32 @@ class ContainerController(object):
         kwargs.setdefault('logger', self.logger)
         return ContainerBroker(db_path, **kwargs)
 
+    def get_and_validate_policy_index(self, req):
+        """
+        Validate that the index supplied maps to a policy.
+
+        :returns: policy index from request, or None if not present
+        :raises: HTTPBadRequest if the supplied index is bogus
+        """
+
+        policy_index = req.headers.get(POLICY_INDEX, None)
+        if policy_index is None:
+            return None
+
+        try:
+            policy_index = int(policy_index)
+        except ValueError:
+            raise HTTPBadRequest(
+                request=req, content_type="text/plain",
+                body=("Invalid X-Storage-Policy-Index %r" % policy_index))
+
+        policy = POLICIES.get_by_index(policy_index)
+        if policy is None:
+            raise HTTPBadRequest(
+                request=req, content_type="text/plain",
+                body=("Invalid X-Storage-Policy-Index %r" % policy_index))
+        return int(policy)
+
     def account_update(self, req, account, container, broker):
         """
         Update the account server(s) with latest container info.
@@ -149,6 +202,7 @@ class ContainerController(object):
                 'x-object-count': info['object_count'],
                 'x-bytes-used': info['bytes_used'],
                 'x-trans-id': req.headers.get('x-trans-id', '-'),
+                POLICY_INDEX: info['storage_policy_index'],
                 'user-agent': 'container-server %s' % os.getpid(),
                 'referer': req.as_referer()})
             if req.headers.get('x-account-override-deleted', 'no').lower() == \
@@ -190,32 +244,32 @@ class ContainerController(object):
         """Handle HTTP DELETE request."""
         drive, part, account, container, obj = split_and_validate_path(
             req, 4, 5, True)
-        if 'x-timestamp' not in req.headers or \
-                not check_float(req.headers['x-timestamp']):
-            return HTTPBadRequest(body='Missing timestamp', request=req,
-                                  content_type='text/plain')
+        req_timestamp = valid_timestamp(req)
         if self.mount_check and not check_mount(self.root, drive):
             return HTTPInsufficientStorage(drive=drive, request=req)
+        # policy index is only relevant for delete_obj (and transitively for
+        # auto create accounts)
+        obj_policy_index = self.get_and_validate_policy_index(req) or 0
         broker = self._get_container_broker(drive, part, account, container)
         if account.startswith(self.auto_create_account_prefix) and obj and \
                 not os.path.exists(broker.db_file):
             try:
-                broker.initialize(normalize_timestamp(
-                    req.headers.get('x-timestamp') or time.time()))
+                broker.initialize(req_timestamp.internal, obj_policy_index)
             except DatabaseAlreadyExists:
                 pass
         if not os.path.exists(broker.db_file):
             return HTTPNotFound()
         if obj:     # delete object
-            broker.delete_object(obj, req.headers.get('x-timestamp'))
+            broker.delete_object(obj, req.headers.get('x-timestamp'),
+                                 obj_policy_index)
             return HTTPNoContent(request=req)
         else:
             # delete container
             if not broker.empty():
                 return HTTPConflict(request=req)
-            existed = float(broker.get_info()['put_timestamp']) and \
+            existed = Timestamp(broker.get_info()['put_timestamp']) and \
                 not broker.is_deleted()
-            broker.delete_db(req.headers['X-Timestamp'])
+            broker.delete_db(req_timestamp.internal)
             if not broker.is_deleted():
                 return HTTPConflict(request=req)
             resp = self.account_update(req, account, container, broker)
@@ -225,19 +279,43 @@ class ContainerController(object):
                 return HTTPNoContent(request=req)
             return HTTPNotFound()
 
-    def _update_or_create(self, req, broker, timestamp):
+    def _update_or_create(self, req, broker, timestamp, new_container_policy,
+                          requested_policy_index):
+        """
+        Create new database broker or update timestamps for existing database.
+
+        :param req: the swob request object
+        :param broker: the broker instance for the container
+        :param timestamp: internalized timestamp
+        :param new_container_policy: the storage policy index to use
+                                     when creating the container
+        :param requested_policy_index: the storage policy index sent in the
+                                       request, may be None
+
+        :returns: created, a bool, if database did not previously exist
+        """
         if not os.path.exists(broker.db_file):
             try:
-                broker.initialize(timestamp)
+                broker.initialize(timestamp, new_container_policy)
             except DatabaseAlreadyExists:
                 pass
             else:
                 return True  # created
-        created = broker.is_deleted()
+        recreated = broker.is_deleted()
+        if recreated:
+            # only set storage policy on deleted containers
+            broker.set_storage_policy_index(new_container_policy,
+                                            timestamp=timestamp)
+        elif requested_policy_index is not None:
+            # validate requested policy with existing container
+            if requested_policy_index != broker.storage_policy_index:
+                raise HTTPConflict(request=req)
         broker.update_put_timestamp(timestamp)
         if broker.is_deleted():
             raise HTTPConflict(request=req)
-        return created
+        if recreated:
+            broker.update_status_changed_at(timestamp)
+        return recreated
 
     @public
     @timing_stats()
@@ -245,10 +323,7 @@ class ContainerController(object):
         """Handle HTTP PUT request."""
         drive, part, account, container, obj = split_and_validate_path(
             req, 4, 5, True)
-        if 'x-timestamp' not in req.headers or \
-                not check_float(req.headers['x-timestamp']):
-            return HTTPBadRequest(body='Missing timestamp', request=req,
-                                  content_type='text/plain')
+        req_timestamp = valid_timestamp(req)
         if 'x-container-sync-to' in req.headers:
             err, sync_to, realm, realm_key = validate_sync_to(
                 req.headers['x-container-sync-to'], self.allowed_sync_hosts,
@@ -257,36 +332,49 @@ class ContainerController(object):
                 return HTTPBadRequest(err)
         if self.mount_check and not check_mount(self.root, drive):
             return HTTPInsufficientStorage(drive=drive, request=req)
-        timestamp = normalize_timestamp(req.headers['x-timestamp'])
+        requested_policy_index = self.get_and_validate_policy_index(req)
         broker = self._get_container_broker(drive, part, account, container)
         if obj:     # put container object
+            # obj put expects the policy_index header, default is for
+            # legacy support during upgrade.
+            obj_policy_index = requested_policy_index or 0
             if account.startswith(self.auto_create_account_prefix) and \
                     not os.path.exists(broker.db_file):
                 try:
-                    broker.initialize(timestamp)
+                    broker.initialize(req_timestamp.internal, obj_policy_index)
                 except DatabaseAlreadyExists:
                     pass
             if not os.path.exists(broker.db_file):
                 return HTTPNotFound()
-            broker.put_object(obj, timestamp, int(req.headers['x-size']),
+            broker.put_object(obj, req_timestamp.internal,
+                              int(req.headers['x-size']),
                               req.headers['x-content-type'],
-                              req.headers['x-etag'])
+                              req.headers['x-etag'], 0,
+                              obj_policy_index)
             return HTTPCreated(request=req)
         else:   # put container
-            created = self._update_or_create(req, broker, timestamp)
+            if requested_policy_index is None:
+                # use the default index sent by the proxy if available
+                new_container_policy = req.headers.get(
+                    'X-Backend-Storage-Policy-Default', int(POLICIES.default))
+            else:
+                new_container_policy = requested_policy_index
+            created = self._update_or_create(req, broker,
+                                             req_timestamp.internal,
+                                             new_container_policy,
+                                             requested_policy_index)
             metadata = {}
             metadata.update(
-                (key, (value, timestamp))
+                (key, (value, req_timestamp.internal))
                 for key, value in req.headers.iteritems()
                 if key.lower() in self.save_headers or
                 is_sys_or_user_meta('container', key))
-            if metadata:
-                if 'X-Container-Sync-To' in metadata:
-                    if 'X-Container-Sync-To' not in broker.metadata or \
-                            metadata['X-Container-Sync-To'][0] != \
-                            broker.metadata['X-Container-Sync-To'][0]:
-                        broker.set_x_container_sync_points(-1, -1)
-                broker.update_metadata(metadata)
+            if 'X-Container-Sync-To' in metadata:
+                if 'X-Container-Sync-To' not in broker.metadata or \
+                        metadata['X-Container-Sync-To'][0] != \
+                        broker.metadata['X-Container-Sync-To'][0]:
+                    broker.set_x_container_sync_points(-1, -1)
+            broker.update_metadata(metadata)
             resp = self.account_update(req, account, container, broker)
             if resp:
                 return resp
@@ -307,15 +395,10 @@ class ContainerController(object):
         broker = self._get_container_broker(drive, part, account, container,
                                             pending_timeout=0.1,
                                             stale_reads_ok=True)
-        if broker.is_deleted():
-            return HTTPNotFound(request=req)
-        info = broker.get_info()
-        headers = {
-            'X-Container-Object-Count': info['object_count'],
-            'X-Container-Bytes-Used': info['bytes_used'],
-            'X-Timestamp': info['created_at'],
-            'X-PUT-Timestamp': info['put_timestamp'],
-        }
+        info, is_deleted = broker.get_info_is_deleted()
+        headers = gen_resp_headers(info, is_deleted=is_deleted)
+        if is_deleted:
+            return HTTPNotFound(request=req, headers=headers)
         headers.update(
             (key, value)
             for key, (value, timestamp) in broker.metadata.iteritems()
@@ -335,16 +418,12 @@ class ContainerController(object):
         :params record: object entry record
         :returns: modified record
         """
-        (name, created, size, content_type, etag) = record
+        (name, created, size, content_type, etag) = record[:5]
         if content_type is None:
             return {'subdir': name}
         response = {'bytes': size, 'hash': etag, 'name': name,
                     'content_type': content_type}
-        last_modified = datetime.utcfromtimestamp(float(created)).isoformat()
-        # python isoformat() doesn't include msecs when zero
-        if len(last_modified) < len("1970-01-01T00:00:00.000000"):
-            last_modified += ".000000"
-        response['last_modified'] = last_modified
+        response['last_modified'] = Timestamp(created).isoformat
         override_bytes_from_content_type(response, logger=self.logger)
         return response
 
@@ -362,37 +441,39 @@ class ContainerController(object):
             return HTTPPreconditionFailed(body='Bad delimiter')
         marker = get_param(req, 'marker', '')
         end_marker = get_param(req, 'end_marker')
-        limit = CONTAINER_LISTING_LIMIT
+        limit = constraints.CONTAINER_LISTING_LIMIT
         given_limit = get_param(req, 'limit')
         if given_limit and given_limit.isdigit():
             limit = int(given_limit)
-            if limit > CONTAINER_LISTING_LIMIT:
+            if limit > constraints.CONTAINER_LISTING_LIMIT:
                 return HTTPPreconditionFailed(
                     request=req,
-                    body='Maximum limit is %d' % CONTAINER_LISTING_LIMIT)
+                    body='Maximum limit is %d'
+                    % constraints.CONTAINER_LISTING_LIMIT)
         out_content_type = get_listing_content_type(req)
         if self.mount_check and not check_mount(self.root, drive):
             return HTTPInsufficientStorage(drive=drive, request=req)
         broker = self._get_container_broker(drive, part, account, container,
                                             pending_timeout=0.1,
                                             stale_reads_ok=True)
-        if broker.is_deleted():
-            return HTTPNotFound(request=req)
-        info = broker.get_info()
-        resp_headers = {
-            'X-Container-Object-Count': info['object_count'],
-            'X-Container-Bytes-Used': info['bytes_used'],
-            'X-Timestamp': info['created_at'],
-            'X-PUT-Timestamp': info['put_timestamp'],
-        }
-        for key, (value, timestamp) in broker.metadata.iteritems():
+        info, is_deleted = broker.get_info_is_deleted()
+        resp_headers = gen_resp_headers(info, is_deleted=is_deleted)
+        if is_deleted:
+            return HTTPNotFound(request=req, headers=resp_headers)
+        container_list = broker.list_objects_iter(
+            limit, marker, end_marker, prefix, delimiter, path,
+            storage_policy_index=info['storage_policy_index'])
+        return self.create_listing(req, out_content_type, info, resp_headers,
+                                   broker.metadata, container_list, container)
+
+    def create_listing(self, req, out_content_type, info, resp_headers,
+                       metadata, container_list, container):
+        for key, (value, timestamp) in metadata.iteritems():
             if value and (key.lower() in self.save_headers or
                           is_sys_or_user_meta('container', key)):
                 resp_headers[key] = value
         ret = Response(request=req, headers=resp_headers,
                        content_type=out_content_type, charset='utf-8')
-        container_list = broker.list_objects_iter(limit, marker, end_marker,
-                                                  prefix, delimiter, path)
         if out_content_type == 'application/json':
             ret.body = json.dumps([self.update_data_record(record)
                                    for record in container_list])
@@ -446,10 +527,7 @@ class ContainerController(object):
     def POST(self, req):
         """Handle HTTP POST request."""
         drive, part, account, container = split_and_validate_path(req, 4)
-        if 'x-timestamp' not in req.headers or \
-                not check_float(req.headers['x-timestamp']):
-            return HTTPBadRequest(body='Missing or bad timestamp',
-                                  request=req, content_type='text/plain')
+        req_timestamp = valid_timestamp(req)
         if 'x-container-sync-to' in req.headers:
             err, sync_to, realm, realm_key = validate_sync_to(
                 req.headers['x-container-sync-to'], self.allowed_sync_hosts,
@@ -461,10 +539,10 @@ class ContainerController(object):
         broker = self._get_container_broker(drive, part, account, container)
         if broker.is_deleted():
             return HTTPNotFound(request=req)
-        timestamp = normalize_timestamp(req.headers['x-timestamp'])
         metadata = {}
         metadata.update(
-            (key, (value, timestamp)) for key, value in req.headers.iteritems()
+            (key, (value, req_timestamp.internal))
+            for key, value in req.headers.iteritems()
             if key.lower() in self.save_headers or
             is_sys_or_user_meta('container', key))
         if metadata:
@@ -503,17 +581,9 @@ class ContainerController(object):
                     'ERROR __call__ error with %(method)s %(path)s '),
                     {'method': req.method, 'path': req.path})
                 res = HTTPInternalServerError(body=traceback.format_exc())
-        trans_time = '%.4f' % (time.time() - start_time)
         if self.log_requests:
-            log_message = '%s - - [%s] "%s %s" %s %s "%s" "%s" "%s" %s' % (
-                req.remote_addr,
-                time.strftime('%d/%b/%Y:%H:%M:%S +0000',
-                              time.gmtime()),
-                req.method, req.path,
-                res.status.split()[0], res.content_length or '-',
-                req.headers.get('x-trans-id', '-'),
-                req.referer or '-', req.user_agent or '-',
-                trans_time)
+            trans_time = time.time() - start_time
+            log_message = get_log_line(req, res, trans_time, '')
             if req.method.upper() == 'REPLICATE':
                 self.logger.debug(log_message)
             else:
