@@ -27,15 +27,17 @@ from eventlet import GreenPool, tpool, Timeout, sleep, hubs
 from eventlet.green import subprocess
 from eventlet.support.greenlets import GreenletExit
 
-from swift.common.ring import Ring
 from swift.common.utils import whataremyips, unlink_older_than, \
-    compute_eta, get_logger, dump_recon_cache, \
+    compute_eta, get_logger, dump_recon_cache, ismount, \
     rsync_ip, mkdirs, config_true_value, list_from_csv, get_hub, \
     tpool_reraise, config_auto_int_value
 from swift.common.bufferedhttp import http_connect
 from swift.common.daemon import Daemon
 from swift.common.http import HTTP_OK, HTTP_INSUFFICIENT_STORAGE
-from swift.obj.diskfile import get_hashes
+from swift.obj import ssync_sender
+from swift.obj.diskfile import (DiskFileManager, get_hashes, get_data_dir,
+                                get_tmp_dir)
+from swift.common.storage_policy import POLICY_INDEX, POLICIES
 
 
 hubs.use_hub(get_hub())
@@ -64,7 +66,6 @@ class ObjectReplicator(Daemon):
         self.port = int(conf.get('bind_port', 6000))
         self.concurrency = int(conf.get('concurrency', 1))
         self.stats_interval = int(conf.get('stats_interval', '300'))
-        self.object_ring = Ring(self.swift_dir, ring_name='object')
         self.ring_check_interval = int(conf.get('ring_check_interval', 15))
         self.next_check = time.time() + self.ring_check_interval
         self.reclaim_age = int(conf.get('reclaim_age', 86400 * 7))
@@ -78,6 +79,11 @@ class ObjectReplicator(Daemon):
         self.recon_cache_path = conf.get('recon_cache_path',
                                          '/var/cache/swift')
         self.rcache = os.path.join(self.recon_cache_path, "object.recon")
+        self.conn_timeout = float(conf.get('conn_timeout', 0.5))
+        self.node_timeout = float(conf.get('node_timeout', 10))
+        self.sync_method = getattr(self, conf.get('sync_method') or 'rsync')
+        self.network_chunk_size = int(conf.get('network_chunk_size', 65536))
+        self.disk_chunk_size = int(conf.get('disk_chunk_size', 65536))
         self.headers = {
             'Content-Length': '0',
             'user-agent': 'obj-replicator %s' % os.getpid()}
@@ -87,6 +93,29 @@ class ObjectReplicator(Daemon):
                                                          False))
         self.handoff_delete = config_auto_int_value(
             conf.get('handoff_delete', 'auto'), 0)
+        self._diskfile_mgr = DiskFileManager(conf, self.logger)
+
+    def sync(self, node, job, suffixes):  # Just exists for doc anchor point
+        """
+        Synchronize local suffix directories from a partition with a remote
+        node.
+
+        :param node: the "dev" entry for the remote node to sync with
+        :param job: information about the partition being synced
+        :param suffixes: a list of suffixes which need to be pushed
+
+        :returns: boolean indicating success or failure
+        """
+        return self.sync_method(node, job, suffixes)
+
+    def get_object_ring(self, policy_idx):
+        """
+        Get the ring object to use to handle a request based on its policy.
+
+        :policy_idx: policy index as defined in swift.conf
+        :returns: appropriate ring object
+        """
+        return POLICIES.get_object_ring(policy_idx, self.swift_dir)
 
     def _rsync(self, args):
         """
@@ -135,14 +164,8 @@ class ObjectReplicator(Daemon):
 
     def rsync(self, node, job, suffixes):
         """
-        Synchronize local suffix directories from a partition with a remote
-        node.
-
-        :param node: the "dev" entry for the remote node to sync with
-        :param job: information about the partition being synced
-        :param suffixes: a list of suffixes which need to be pushed
-
-        :returns: boolean indicating success or failure
+        Uses rsync to implement the sync method. This was the first
+        sync method in Swift.
         """
         if not os.path.exists(job['path']):
             return False
@@ -171,19 +194,24 @@ class ObjectReplicator(Daemon):
                 had_any = True
         if not had_any:
             return False
+        data_dir = get_data_dir(job['policy_idx'])
         args.append(join(rsync_module, node['device'],
-                    'objects', job['partition']))
+                    data_dir, job['partition']))
         return self._rsync(args) == 0
 
-    def check_ring(self):
+    def ssync(self, node, job, suffixes):
+        return ssync_sender.Sender(self, node, job, suffixes)()
+
+    def check_ring(self, object_ring):
         """
         Check to see if the ring has been updated
+        :param object_ring: the ring to check
 
         :returns: boolean indicating whether or not the ring has changed
         """
         if time.time() > self.next_check:
             self.next_check = time.time() + self.ring_check_interval
-            if self.object_ring.has_changed():
+            if object_ring.has_changed():
                 return False
         return True
 
@@ -200,13 +228,14 @@ class ObjectReplicator(Daemon):
                     if len(suff) == 3 and isdir(join(path, suff))]
         self.replication_count += 1
         self.logger.increment('partition.delete.count.%s' % (job['device'],))
+        self.headers[POLICY_INDEX] = job['policy_idx']
         begin = time.time()
         try:
             responses = []
             suffixes = tpool.execute(tpool_get_suffixes, job['path'])
             if suffixes:
                 for node in job['nodes']:
-                    success = self.rsync(node, job, suffixes)
+                    success = self.sync(node, job, suffixes)
                     if success:
                         with Timeout(self.http_timeout):
                             conn = http_connect(
@@ -241,6 +270,7 @@ class ObjectReplicator(Daemon):
         """
         self.replication_count += 1
         self.logger.increment('partition.update.count.%s' % (job['device'],))
+        self.headers[POLICY_INDEX] = job['policy_idx']
         begin = time.time()
         try:
             hashed, local_hash = tpool_reraise(
@@ -252,7 +282,7 @@ class ObjectReplicator(Daemon):
             attempts_left = len(job['nodes'])
             nodes = itertools.chain(
                 job['nodes'],
-                self.object_ring.get_more_nodes(int(job['partition'])))
+                job['object_ring'].get_more_nodes(int(job['partition'])))
             while attempts_left > 0:
                 # If this throws StopIterator it will be caught way below
                 node = next(nodes)
@@ -290,7 +320,7 @@ class ObjectReplicator(Daemon):
                     suffixes = [suffix for suffix in local_hash if
                                 local_hash[suffix] !=
                                 remote_hash.get(suffix, -1)]
-                    self.rsync(node, job, suffixes)
+                    self.sync(node, job, suffixes)
                     with Timeout(self.http_timeout):
                         conn = http_connect(
                             node['replication_ip'], node['replication_port'],
@@ -377,20 +407,20 @@ class ObjectReplicator(Daemon):
                 self.kill_coros()
             self.last_replication_count = self.replication_count
 
-    def collect_jobs(self):
+    def process_repl(self, policy, jobs, ips):
         """
-        Returns a sorted list of jobs (dictionaries) that specify the
-        partitions, nodes, etc to be rsynced.
+        Helper function for collect_jobs to build jobs for replication
+        using replication style storage policy
         """
-        jobs = []
-        ips = whataremyips()
-        for local_dev in [dev for dev in self.object_ring.devs
+        obj_ring = self.get_object_ring(policy.idx)
+        data_dir = get_data_dir(policy.idx)
+        for local_dev in [dev for dev in obj_ring.devs
                           if dev and dev['replication_ip'] in ips and
                           dev['replication_port'] == self.port]:
             dev_path = join(self.devices_dir, local_dev['device'])
-            obj_path = join(dev_path, 'objects')
-            tmp_path = join(dev_path, 'tmp')
-            if self.mount_check and not os.path.ismount(dev_path):
+            obj_path = join(dev_path, data_dir)
+            tmp_path = join(dev_path, get_tmp_dir(int(policy)))
+            if self.mount_check and not ismount(dev_path):
                 self.logger.warn(_('%s is not mounted'), local_dev['device'])
                 continue
             unlink_older_than(tmp_path, time.time() - self.reclaim_age)
@@ -406,12 +436,12 @@ class ObjectReplicator(Daemon):
                     if isfile(job_path):
                         # Clean up any (probably zero-byte) files where a
                         # partition should be.
-                        self.logger.warning('Removing partition directory '
-                                            'which was a file: %s', job_path)
+                        self.logger.warning(
+                            'Removing partition directory '
+                            'which was a file: %s', job_path)
                         os.remove(job_path)
                         continue
-                    part_nodes = \
-                        self.object_ring.get_part_nodes(int(partition))
+                    part_nodes = obj_ring.get_part_nodes(int(partition))
                     nodes = [node for node in part_nodes
                              if node['id'] != local_dev['id']]
                     jobs.append(
@@ -419,9 +449,23 @@ class ObjectReplicator(Daemon):
                              device=local_dev['device'],
                              nodes=nodes,
                              delete=len(nodes) > len(part_nodes) - 1,
-                             partition=partition))
+                             policy_idx=policy.idx,
+                             partition=partition,
+                             object_ring=obj_ring))
+
                 except (ValueError, OSError):
                     continue
+
+    def collect_jobs(self):
+        """
+        Returns a sorted list of jobs (dictionaries) that specify the
+        partitions, nodes, etc to be rsynced.
+        """
+        jobs = []
+        ips = whataremyips()
+        for policy in POLICIES:
+            # may need to branch here for future policy types
+            self.process_repl(policy, jobs, ips)
         random.shuffle(jobs)
         if self.handoffs_first:
             # Move the handoff parts to the front of the list
@@ -458,10 +502,10 @@ class ObjectReplicator(Daemon):
                         job['partition'] not in override_partitions:
                     continue
                 dev_path = join(self.devices_dir, job['device'])
-                if self.mount_check and not os.path.ismount(dev_path):
+                if self.mount_check and not ismount(dev_path):
                     self.logger.warn(_('%s is not mounted'), job['device'])
                     continue
-                if not self.check_ring():
+                if not self.check_ring(job['object_ring']):
                     self.logger.info(_("Ring change detected. Aborting "
                                        "current replication pass."))
                     return
@@ -509,6 +553,6 @@ class ObjectReplicator(Daemon):
             dump_recon_cache({'object_replication_time': total,
                               'object_replication_last': time.time()},
                              self.rcache, self.logger)
-            self.logger.debug(_('Replication sleeping for %s seconds.'),
+            self.logger.debug('Replication sleeping for %s seconds.',
                               self.run_pause)
             sleep(self.run_pause)
