@@ -25,6 +25,8 @@ from swift.common import http
 from swift.common import swob
 from swift.common import utils
 
+from swift.common.storage_policy import POLICY_INDEX
+
 
 class Receiver(object):
     """
@@ -168,6 +170,7 @@ class Receiver(object):
         self.request.environ['eventlet.minimum_write_chunk_size'] = 0
         self.device, self.partition = utils.split_path(
             urllib.unquote(self.request.path), 2, 2, False)
+        self.policy_idx = int(self.request.headers.get(POLICY_INDEX, 0))
         utils.validate_device_partition(self.device, self.partition)
         if self.app._diskfile_mgr.mount_check and \
                 not constraints.check_mount(
@@ -228,7 +231,7 @@ class Receiver(object):
             want = False
             try:
                 df = self.app._diskfile_mgr.get_diskfile_from_hash(
-                    self.device, self.partition, object_hash)
+                    self.device, self.partition, object_hash, self.policy_idx)
             except exceptions.DiskFileNotExist:
                 want = True
             else:
@@ -236,7 +239,7 @@ class Receiver(object):
                     df.open()
                 except exceptions.DiskFileDeleted as err:
                     want = err.timestamp < timestamp
-                except exceptions.DiskFileError, err:
+                except exceptions.DiskFileError as err:
                     want = True
                 else:
                     want = df.timestamp < timestamp
@@ -351,6 +354,7 @@ class Receiver(object):
                     subreq_iter())
             else:
                 raise Exception('Invalid subrequest method %s' % method)
+            subreq.headers[POLICY_INDEX] = self.policy_idx
             subreq.headers['X-Backend-Replication'] = 'True'
             if replication_headers:
                 subreq.headers['X-Backend-Replication-Headers'] = \
