@@ -19,9 +19,16 @@ from StringIO import StringIO
 import unittest
 from urllib import quote
 import zlib
+from textwrap import dedent
+import os
 
+from test.unit import FakeLogger
 from eventlet.green import urllib2
 from swift.common import internal_client
+from swift.common import swob
+
+from test.unit import with_tempdir, write_fake_ring, patch_policies
+from test.unit.common.middleware.helpers import FakeSwift
 
 
 def not_sleep(seconds):
@@ -48,6 +55,21 @@ def make_path(account, container=None, obj=None):
     return path
 
 
+def make_path_info(account, container=None, obj=None):
+    # FakeSwift keys on PATH_INFO - which is *encoded* but unquoted
+    path = '/v1/%s' % '/'.join(
+        p for p in (account, container, obj) if p)
+    return path.encode('utf-8')
+
+
+def get_client_app():
+    app = FakeSwift()
+    with mock.patch('swift.common.internal_client.loadapp',
+                    new=lambda *args, **kwargs: app):
+        client = internal_client.InternalClient({}, 'test', 1)
+    return client, app
+
+
 class InternalClient(internal_client.InternalClient):
     def __init__(self):
         pass
@@ -62,7 +84,8 @@ class GetMetadataInternalClient(internal_client.InternalClient):
         self.get_metadata_called = 0
         self.metadata = 'some_metadata'
 
-    def _get_metadata(self, path, metadata_prefix, acceptable_statuses=None):
+    def _get_metadata(self, path, metadata_prefix, acceptable_statuses=None,
+                      headers=None):
         self.get_metadata_called += 1
         self.test.assertEquals(self.path, path)
         self.test.assertEquals(self.metadata_prefix, metadata_prefix)
@@ -178,6 +201,52 @@ class TestCompressingfileReader(unittest.TestCase):
 
 
 class TestInternalClient(unittest.TestCase):
+
+    @patch_policies(legacy_only=True)
+    @mock.patch('swift.common.utils.HASH_PATH_SUFFIX', new='endcap')
+    @with_tempdir
+    def test_load_from_config(self, tempdir):
+        conf_path = os.path.join(tempdir, 'interal_client.conf')
+        conf_body = """
+        [DEFAULT]
+        swift_dir = %s
+
+        [pipeline:main]
+        pipeline = catch_errors cache proxy-server
+
+        [app:proxy-server]
+        use = egg:swift#proxy
+        auto_create_account_prefix = -
+
+        [filter:cache]
+        use = egg:swift#memcache
+
+        [filter:catch_errors]
+        use = egg:swift#catch_errors
+        """ % tempdir
+        with open(conf_path, 'w') as f:
+            f.write(dedent(conf_body))
+        account_ring_path = os.path.join(tempdir, 'account.ring.gz')
+        write_fake_ring(account_ring_path)
+        container_ring_path = os.path.join(tempdir, 'container.ring.gz')
+        write_fake_ring(container_ring_path)
+        object_ring_path = os.path.join(tempdir, 'object.ring.gz')
+        write_fake_ring(object_ring_path)
+        client = internal_client.InternalClient(conf_path, 'test', 1)
+        self.assertEqual(client.account_ring, client.app.app.app.account_ring)
+        self.assertEqual(client.account_ring.serialized_path,
+                         account_ring_path)
+        self.assertEqual(client.container_ring,
+                         client.app.app.app.container_ring)
+        self.assertEqual(client.container_ring.serialized_path,
+                         container_ring_path)
+        object_ring = client.app.app.app.get_object_ring(0)
+        self.assertEqual(client.get_object_ring(0),
+                         object_ring)
+        self.assertEqual(object_ring.serialized_path,
+                         object_ring_path)
+        self.assertEquals(client.auto_create_account_prefix, '-')
+
     def test_init(self):
         class App(object):
             def __init__(self, test, conf_path):
@@ -427,21 +496,24 @@ class TestInternalClient(unittest.TestCase):
         self.assertEquals(1, client.make_request_called)
 
     def test_get_metadata_invalid_status(self):
-        class Response(object):
-            def __init__(self):
-                self.status_int = 404
-                self.headers = {'some_key': 'some_value'}
+        class FakeApp(object):
+
+            def __call__(self, environ, start_response):
+                start_response('404 Not Found', [('x-foo', 'bar')])
+                return ['nope']
 
         class InternalClient(internal_client.InternalClient):
             def __init__(self):
-                pass
-
-            def make_request(self, *a, **kw):
-                return Response()
+                self.user_agent = 'test'
+                self.request_tries = 1
+                self.app = FakeApp()
 
         client = InternalClient()
-        metadata = client._get_metadata('path')
-        self.assertEquals({}, metadata)
+        self.assertRaises(internal_client.UnexpectedResponse,
+                          client._get_metadata, 'path')
+        metadata = client._get_metadata('path', metadata_prefix='x-',
+                                        acceptable_statuses=(4,))
+        self.assertEqual(metadata, {'foo': 'bar'})
 
     def test_make_path(self):
         account, container, obj = path_parts()
@@ -652,6 +724,26 @@ class TestInternalClient(unittest.TestCase):
         self.assertEquals(client.metadata, metadata)
         self.assertEquals(1, client.get_metadata_called)
 
+    def test_get_metadadata_with_acceptable_status(self):
+        account, container, obj = path_parts()
+        path = make_path_info(account)
+        client, app = get_client_app()
+        resp_headers = {'some-important-header': 'some value'}
+        app.register('GET', path, swob.HTTPOk, resp_headers)
+        metadata = client.get_account_metadata(
+            account, acceptable_statuses=(2, 4))
+        self.assertEqual(metadata['some-important-header'],
+                         'some value')
+        app.register('GET', path, swob.HTTPNotFound, resp_headers)
+        metadata = client.get_account_metadata(
+            account, acceptable_statuses=(2, 4))
+        self.assertEqual(metadata['some-important-header'],
+                         'some value')
+        app.register('GET', path, swob.HTTPServerError, resp_headers)
+        self.assertRaises(internal_client.UnexpectedResponse,
+                          client.get_account_metadata, account,
+                          acceptable_statuses=(2, 4))
+
     def test_set_account_metadata(self):
         account, container, obj = path_parts()
         path = make_path(account)
@@ -822,6 +914,47 @@ class TestInternalClient(unittest.TestCase):
         self.assertEquals(client.metadata, metadata)
         self.assertEquals(1, client.get_metadata_called)
 
+    def test_get_metadata_extra_headers(self):
+        class InternalClient(internal_client.InternalClient):
+            def __init__(self):
+                self.app = self.fake_app
+                self.user_agent = 'some_agent'
+                self.request_tries = 3
+
+            def fake_app(self, env, start_response):
+                self.req_env = env
+                start_response('200 Ok', [('Content-Length', '0')])
+                return []
+
+        client = InternalClient()
+        headers = {'X-Foo': 'bar'}
+        client.get_object_metadata('account', 'container', 'obj',
+                                   headers=headers)
+        self.assertEqual(client.req_env['HTTP_X_FOO'], 'bar')
+
+    def test_get_object(self):
+        account, container, obj = path_parts()
+        path_info = make_path_info(account, container, obj)
+        client, app = get_client_app()
+        headers = {'foo': 'bar'}
+        body = 'some_object_body'
+        app.register('GET', path_info, swob.HTTPOk, headers, body)
+        req_headers = {'x-important-header': 'some_important_value'}
+        status_int, resp_headers, obj_iter = client.get_object(
+            account, container, obj, req_headers)
+        self.assertEqual(status_int // 100, 2)
+        for k, v in headers.items():
+            self.assertEqual(v, resp_headers[k])
+        self.assertEqual(''.join(obj_iter), body)
+        self.assertEqual(resp_headers['content-length'], str(len(body)))
+        self.assertEqual(app.call_count, 1)
+        req_headers.update({
+            'host': 'localhost:80',  # from swob.Request.blank
+            'user-agent': 'test',   # from InternalClient.make_request
+        })
+        self.assertEqual(app.calls_with_headers, [(
+            'GET', path_info, swob.HeaderKeyDict(req_headers))])
+
     def test_iter_object_lines(self):
         class InternalClient(internal_client.InternalClient):
             def __init__(self, lines):
@@ -922,6 +1055,33 @@ class TestInternalClient(unittest.TestCase):
         client.upload_object(fobj, account, container, obj, headers)
         self.assertEquals(1, client.make_request_called)
 
+    def test_upload_object_not_chunked(self):
+        class InternalClient(internal_client.InternalClient):
+            def __init__(self, test, path, headers, fobj):
+                self.test = test
+                self.path = path
+                self.headers = headers
+                self.fobj = fobj
+                self.make_request_called = 0
+
+            def make_request(
+                    self, method, path, headers, acceptable_statuses,
+                    body_file=None):
+                self.make_request_called += 1
+                self.test.assertEquals(self.path, path)
+                exp_headers = dict(self.headers)
+                self.test.assertEquals(exp_headers, headers)
+                self.test.assertEquals(self.fobj, fobj)
+
+        fobj = 'some_fobj'
+        account, container, obj = path_parts()
+        path = make_path(account, container, obj)
+        headers = {'key': 'value', 'Content-Length': len(fobj)}
+
+        client = InternalClient(self, path, headers, fobj)
+        client.upload_object(fobj, account, container, obj, headers)
+        self.assertEquals(1, client.make_request_called)
+
 
 class TestGetAuth(unittest.TestCase):
     @mock.patch('eventlet.green.urllib2.urlopen')
@@ -946,21 +1106,38 @@ class TestGetAuth(unittest.TestCase):
                           'http://127.0.0.1', 'user', 'key', auth_version=2.0)
 
 
+mock_time_value = 1401224049.98
+
+
+def mock_time():
+    global mock_time_value
+    mock_time_value += 1
+    return mock_time_value
+
+
 class TestSimpleClient(unittest.TestCase):
 
     @mock.patch('eventlet.green.urllib2.urlopen')
     @mock.patch('eventlet.green.urllib2.Request')
+    @mock.patch('swift.common.internal_client.time', mock_time)
     def test_get(self, request, urlopen):
         # basic GET request, only url as kwarg
         request.return_value.get_type.return_value = "http"
         urlopen.return_value.read.return_value = ''
+        urlopen.return_value.getcode.return_value = 200
+        urlopen.return_value.info.return_value = {'content-length': '345'}
         sc = internal_client.SimpleClient(url='http://127.0.0.1')
-        retval = sc.retry_request('GET')
+        logger = FakeLogger()
+        retval = sc.retry_request(
+            'GET', headers={'content-length': '123'}, logger=logger)
         request.assert_called_with('http://127.0.0.1?format=json',
-                                   headers={},
+                                   headers={'content-length': '123'},
                                    data=None)
         self.assertEqual([None, None], retval)
         self.assertEqual('GET', request.return_value.get_method())
+        self.assertEqual(logger.log_dict['debug'], [(
+            ('-> 2014-05-27T20:54:11 GET http://127.0.0.1%3Fformat%3Djson 200 '
+             '123 345 1401224050.98 1401224051.98 1.0 -',), {})])
 
         # Check if JSON is decoded
         urlopen.return_value.read.return_value = '{}'
@@ -994,7 +1171,7 @@ class TestSimpleClient(unittest.TestCase):
 
         # same as above, now with object name
         retval = sc.retry_request('GET', container='cont', name='obj')
-        request.assert_called_with('http://127.0.0.1/cont/obj?format=json',
+        request.assert_called_with('http://127.0.0.1/cont/obj',
                                    headers={'X-Auth-Token': 'token'},
                                    data=None)
         self.assertEqual([None, {}], retval)
@@ -1018,7 +1195,8 @@ class TestSimpleClient(unittest.TestCase):
         urlopen.return_value.read.return_value = ''
         req = urllib2.Request('http://127.0.0.1', method='GET')
         request.side_effect = [urllib2.URLError(''), req]
-        sc = internal_client.SimpleClient(url='http://127.0.0.1', retries=1)
+        sc = internal_client.SimpleClient(url='http://127.0.0.1', retries=1,
+                                          token='token')
 
         retval = sc.retry_request('GET')
         self.assertEqual(request.call_count, 3)
